package picketlink.test.axis2;

import java.net.URI;
import java.util.HashMap;
import java.util.Map;
import java.util.UUID;

import javax.activation.DataHandler;
import javax.xml.namespace.QName;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.apache.axiom.om.OMAbstractFactory;
import org.apache.axiom.om.OMElement;
import org.apache.axiom.soap.SOAP11Constants;
import org.apache.axiom.soap.SOAPBody;
import org.apache.axiom.soap.SOAPEnvelope;
import org.apache.axiom.soap.SOAPFactory;
import org.apache.axis2.AxisFault;
import org.apache.axis2.Constants;
import org.apache.axis2.addressing.AddressingConstants;
import org.apache.axis2.addressing.EndpointReference;
import org.apache.axis2.client.OperationClient;
import org.apache.axis2.client.Options;
import org.apache.axis2.client.ServiceClient;
import org.apache.axis2.context.MessageContext;
import org.apache.axis2.transport.http.HTTPConstants;
import org.apache.axis2.util.XMLUtils;
import org.apache.axis2.wsdl.WSDLConstants;
import org.apache.commons.httpclient.DefaultHttpMethodRetryHandler;
import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.HttpConnectionManager;
import org.apache.commons.httpclient.MultiThreadedHttpConnectionManager;
import org.apache.commons.httpclient.params.HttpMethodParams;
import org.apache.commons.io.output.ByteArrayOutputStream;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

public class Axis2Sender {

	public static final String WS_TRUST_NS_URL = "http://docs.oasis-open.org/ws-sx/ws-trust/200512";
	/**
	 * 
	 */
	private HttpClient httpClient;

	private Options mOptions;

	/**
	 * SOAP Factory for the respective types
	 */
	private SOAPFactory soapFactory;

	/**
	 * Apache Axis2 client sender mechanism
	 */
	private ServiceClient mSender;

	public Axis2Sender() {
		mOptions = new Options();
		initializeHttpClient();

		try {
			// Initialize Axis2 Service Sender
			mSender = new ServiceClient();
			mOptions
					.setSoapVersionURI(SOAP11Constants.SOAP_ENVELOPE_NAMESPACE_URI);
			mSender.setOptions(mOptions);

			soapFactory = OMAbstractFactory.getSOAP11Factory();

		} catch (AxisFault e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	protected void initializeHttpClient() {
		HttpConnectionManager connManager = new MultiThreadedHttpConnectionManager();
		httpClient = new HttpClient(connManager);

		DefaultHttpMethodRetryHandler retryhandler = new DefaultHttpMethodRetryHandler(
				0, true);
		httpClient.getParams().setParameter(HttpMethodParams.RETRY_HANDLER,
				retryhandler);

		// Set the HTTP Client instance to use in sending
		mOptions.setProperty(HTTPConstants.REUSE_HTTP_CLIENT, Boolean.TRUE);
		mOptions.setProperty(HTTPConstants.CACHED_HTTP_CLIENT, httpClient);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.openhealthtools.ihe.common.ws.OHFSOAPSender#createEnvelope()
	 */
	public SOAPEnvelope createEnvelope() {
		SOAPEnvelope envelope = soapFactory.createSOAPEnvelope();
		soapFactory.createSOAPBody(envelope);
		soapFactory.createSOAPHeader(envelope);
		envelope.build();
		return envelope;
	}

	public Element send(String pStsUrl, Element pRequestBody,
			Element pRequestHeader, String pAction) throws Exception {
		OMElement bodyOMElement = XMLUtils.toOM(pRequestBody);
		// Create SOAP Envelope
		SOAPEnvelope requestEnvelope = createEnvelope();

		// Finalize body for send
		SOAPBody requestBody = requestEnvelope.getBody();
		requestBody.addChild(bodyOMElement);

		requestEnvelope.getHeader().addChild(XMLUtils.toOM(pRequestHeader));

		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		requestEnvelope.serialize(baos);
		String serializedRequestBody = baos.toString();

		System.out.println("****** XUA SOAP REQUEST ******");
		System.out.println(serializedRequestBody);
		System.out.println("********************************");
		

		// Do Send
		SOAPEnvelope responseEnvelope = send(new java.net.URI(pStsUrl),
				requestEnvelope,
				new HashMap<String, javax.activation.DataHandler>(), pAction);

		if (null == responseEnvelope) {
			// sLogger.error(XDSSoapConstants.NULL_SOAP_ENVELOPE);
			throw new Exception("");
		}
		// Verify the response body is not null
		SOAPBody responseBody = responseEnvelope.getBody();
		if (null == responseBody) {
			// sLogger.error(XDSSoapConstants.NULL_SOAP_BODY);
			throw new Exception("");
		}

		// Get Response Element
		OMElement responseOMElement = responseEnvelope.getBody()
				.getFirstElement();
		if (responseOMElement == null) {
			// sLogger.error(XDSSoapConstants.EMPTY_RESPONSE);
			throw new Exception("");
		}
		OMElement assertion = null;
		do {
			OMElement rst = responseOMElement.getFirstChildWithName(new QName(
					WS_TRUST_NS_URL, "RequestedSecurityToken"));
			if (null == rst) {
				break;
			}
			assertion = rst.getFirstChildWithName(new QName(
					"urn:oasis:names:tc:SAML:2.0:assertion", "Assertion"));
		} while (false);
		if (assertion == null) {
			throw new Exception("could not parse assertion.");
		}
		// We have a response with content. Serialize content to be human
		// readable
		baos = new ByteArrayOutputStream();
		assertion.serialize(baos);
		String serializedResponseBody = baos.toString();

		System.out.println("****** XUA SOAP RESPONSE BODY ******");
		System.out
				.println("messaged removed - otherwise it flushes the buffer, see wire logs.");
		System.out.println("********************************");

		System.out.println("Ending XDS SOAP Processing");

		System.out.println(serializedResponseBody);

		// Message
		return XMLUtils.toDOM(assertion);

	}

	private SOAPEnvelope send(URI uri, SOAPEnvelope requestEnvelope,
			HashMap<String, DataHandler> hashMap, String action)
			throws AxisFault, Exception {

		// Prepare message context for sending
		MessageContext requestMessageContext = prepareMessageContext(uri,
				requestEnvelope, action, hashMap);

		// Send MC
		MessageContext responseMessageContext = executeSend(requestMessageContext);

		SOAPEnvelope responseEnvelope = responseMessageContext.getEnvelope();

		return responseEnvelope;
	}

	/**
	 * Finalizes construction of sockets and sends a SOAP Message over a given
	 * ATNA socket.
	 * 
	 * @throws IHESOAPException
	 */
	private MessageContext executeSend(MessageContext requestMessageContext)
			throws Exception {

		OperationClient sendClient = mSender
				.createClient(ServiceClient.ANON_OUT_IN_OP);

		sendClient.addMessageContext(requestMessageContext);
		requestMessageContext.setProperty(
				AddressingConstants.REPLACE_ADDRESSING_HEADERS,
				Constants.VALUE_FALSE);
		requestMessageContext.setProperty(
				AddressingConstants.INCLUDE_OPTIONAL_HEADERS,
				Constants.VALUE_TRUE);

		String action = requestMessageContext.getOptions().getAction();
		if (action == null || action.equals("")) {
			requestMessageContext.setProperty(
					AddressingConstants.DISABLE_ADDRESSING_FOR_OUT_MESSAGES,
					Constants.VALUE_TRUE);
		} else {
			requestMessageContext.setProperty(
					AddressingConstants.DISABLE_ADDRESSING_FOR_OUT_MESSAGES,
					Constants.VALUE_FALSE);
		}
		sendClient.execute(true);

		return sendClient
				.getMessageContext(WSDLConstants.MESSAGE_LABEL_IN_VALUE);
	}

	private MessageContext prepareMessageContext(URI endpoint, SOAPEnvelope envelope, String action, Map<String,javax.activation.DataHandler> attachments) throws Exception,AxisFault
	{
		if (null == endpoint || endpoint.toString().equals("")) {
			throw new Exception("No Service Endpoint Defined");
		}
		
		// Create MC
		MessageContext requestMessageContext = new MessageContext();
		requestMessageContext.setEnvelope(envelope);
		requestMessageContext.setOptions(mOptions);
		if (!(action == null || action.equals(""))) {
			requestMessageContext.getOptions().setAction(action);
			requestMessageContext.getOptions().setMessageId("urn:uuid:"+UUID.randomUUID());
		}			
		
		// Set destination endpiont uri
		requestMessageContext.getOptions().setTo(new EndpointReference(endpoint.toString()));
		
		// Push service context into Message Context
		requestMessageContext.setServiceContext(mSender.getServiceContext());
		
		return requestMessageContext;
	}
	
}
