package picketlink.test.axis2;

import java.io.ByteArrayInputStream;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

public class WSTrustClientAxis2Test 
{
    private static final org.apache.log4j.Logger sLogger = org.apache.log4j.Logger.getLogger(WSTrustClientAxis2Test.class);

    private Axis2Sender axisSender = new Axis2Sender();
    
    public static final String WS_TRUST_NS_URL = "http://docs.oasis-open.org/ws-sx/ws-trust/200512";
    
    /**
     * WS Action for XUser
     */
    public static final String WS_ACTION_XUSER = WS_TRUST_NS_URL + "/RST/Issue";
    
    /**
     * WS-Trust Token Request body template
     */
    public static final String WS_TRUST_TOKEN_REQUEST_BODY = 
        "<wst:RequestSecurityToken xmlns:wst=\""+WS_TRUST_NS_URL+"\">" +
        "<wst:RequestType>"+WS_TRUST_NS_URL+"/Issue</wst:RequestType>" +
        "<wsp:AppliesTo xmlns:wsp=\"http://schemas.xmlsoap.org/ws/2004/09/policy\">" +
        "<wsa:EndpointReference xmlns:wsa=\"http://www.w3.org/2005/08/addressing\">"+
        "<wsa:Address>http://localhost:8080/picketlink-sts/PicketLinkSTS</wsa:Address>"+
        "</wsa:EndpointReference>"+
        "</wsp:AppliesTo>"+
        "<wst:Claims Dialect=\"urn:ibm:names:ITFIM:saml\">"+
        "<fimsaml:Saml20Claims xmlns:fimsaml=\"urn:ibm:names:ITFIM:saml\">"+
        "<fimsaml:ConfirmationMethod>urn:oasis:names:tc:SAML:2.0:cm:bearer</fimsaml:ConfirmationMethod>"+
        "<saml:NameID xmlns:saml=\"urn:oasis:names:tc:SAML:2.0:assertion\" Format=\"urn:oasis:names:tc:SAML:1.1:nameid-format:emailAddress\">admin</saml:NameID>"+
        "</fimsaml:Saml20Claims>"+
        "</wst:Claims>"+
        "</wst:RequestSecurityToken>";    

    /**
     * WS-Trust Token Request header template
     */
    public static final String WS_TRUST_TOKEN_REQUEST_HEADER = 
        "<wsse:Security xmlns:wsse=\"http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd\">"+
        "<wsse:UsernameToken>"+
        "<wsse:Username>admin</wsse:Username>"+
        "<wsse:Password Type='http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-username-token-profile-1.0#PasswordText'>admin</wsse:Password>"+
        "</wsse:UsernameToken>"+
        "</wsse:Security>";
    
    public WSTrustClientAxis2Test()
    {
    }
   
    private Element buildWsTrustRequestBody(String pUserName, String pServiceUri) throws Exception
    {
        return loadIntoXML(WS_TRUST_TOKEN_REQUEST_BODY);        
    }
    
    public static Element loadIntoXML(String content)
    {
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        Element elm = null;
        try
        {
            factory.setNamespaceAware(true);
            DocumentBuilder builder = factory.newDocumentBuilder();
            Document doc = builder.parse(new ByteArrayInputStream(content.getBytes()));
            elm = doc.getDocumentElement();
        }
        catch (Exception ex)
        {
            sLogger.error("got error " + ex.toString(), ex);
        }
        return elm;
    }    
    
    private Element buildWsTrustRequestHeader(String pUserName, String pPassword, String pServiceUri) throws Exception
    {
        return loadIntoXML(WS_TRUST_TOKEN_REQUEST_HEADER);        
    }
    
    
    public Element getTokenFromSts(String pStsUrl, String pServiceUri, String pUserName, String pPassword) throws Exception
    {
        if (null == pUserName || null == pPassword || null == pServiceUri) {
            throw new IllegalArgumentException("You must specify a username, password, and service URI to use XUA.");
        }
        
        Element response = null;
        try
        {
            Element elementBody, elementHeader;
            elementBody = buildWsTrustRequestBody(pUserName, pServiceUri);
            elementHeader = buildWsTrustRequestHeader(pUserName, pPassword, pServiceUri);
            response = retrieveToken(pStsUrl, elementBody, elementHeader);
            
        }
        catch (Exception ex)
        {
            sLogger.error("Error getting assertion from STS: " + ex.getMessage(), ex);
            throw ex;
        }
        return response;        
    }

    private Element retrieveToken(String pStsUrl, Element pWsTrustRequestBody, Element pWsTrustRequestHeader) throws Exception
    {
        if (null == pStsUrl) {
            throw new IllegalArgumentException("No Security Token Service defined.");
        }
        if (sLogger.isDebugEnabled()) {
            sLogger.debug(">>>> get SAML token from STS <<<<");
        }
        Element response = axisSender.send(pStsUrl, pWsTrustRequestBody, pWsTrustRequestHeader, WS_ACTION_XUSER);
       
        return response;
    }
    
    
	
	
	public static void main(String[] args)
    {
        String pStsUrl="http://localhost:8080/picketlink-sts/";
        String pServiceUri="https://10.240.60.12:8011/axis2/services/xdsregistryb";
        String pUserName="";
        String pPassword="";
        try {
			Element tokenFromSts = new WSTrustClientAxis2Test().getTokenFromSts(pStsUrl, pServiceUri, pUserName, pPassword);
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}		

    }
}
